from unittest import TestCase

import textarium.preprocessing as preprocessing

class TestPreprocessing(TestCase):
    def test_remove_extra_spaces_0(self):
        test_input = " This line    has a lot of   extra spaces      "
        expected_result = "This line has a lot of extra spaces"
        result = preprocessing.remove_extra_spaces(test_input)
        self.assertEqual(expected_result, result)

    def test_remove_charset_0(self):
        test_input = "This 3line has0 !extra chars-"
        expected_result = "This line has extra chars"
        result = preprocessing.remove_charset(test_input, charset="30!-")
        self.assertEqual(expected_result, result)

    def test_remove_html_0(self):
        test_input = "<div>This line contains HTML-tags&nbsp</div>"
        expected_result = "This line contains HTML-tags"
        result = preprocessing.remove_html(test_input)
        self.assertEqual(expected_result, result)

    def test_remove_url_0(self):
        test_input = """
        This line contains URL http://google.com and 
        https://amazon.com https://yandex.ru/items?url=1
        """
        expected_result = "This line contains URL and"
        result = preprocessing.remove_urls(test_input)
        self.assertEqual(expected_result, result)

    def test_remove_words_0(self):
        test_input = "This dog line contains cat extra mouse word bird tokens"
        words_to_exclude = ["dog", "cat", "mouse", "bird"]
        expected_result = "This line contains extra word tokens"
        result = preprocessing.remove_words(test_input, words_to_exclude)
        self.assertEqual(expected_result, result)

    def test_remove_digits_0(self):
        test_input = "This 3 line has 1234 random digits-10."
        expected_result = "This line has random digits- ."
        result = preprocessing.remove_digits(test_input)
        self.assertEqual(expected_result, result)

    def test_remove_punctuation_0(self):
        test_input = "Hello! I ne-ed remove, all : punctuation."
        expected_result = "Hello I ne ed remove all punctuation"
        result = preprocessing.remove_punctuation(test_input)
        self.assertEqual(expected_result, result)

    def test_lemmatize_ru_0(self):
        test_input = "эти дети скоро пойдут в школу и выучат английский и немецкий языки"
        expected_result = "этот ребёнок скоро пойти в школа и выучить английский и немецкий язык"
        result = preprocessing.lemmatize(test_input, lang='ru')
        self.assertEqual(expected_result, result)

    def test_lemmatize_en_0(self):
        test_input = "These children are doing very dangerous things"
        expected_result = "These child are doing very dangerous thing"
        result = preprocessing.lemmatize(test_input, lang='en')
        self.assertEqual(expected_result, result)



# -*- coding: utf-8 -*-

"""
Text cleaning and preprocessing functions.
"""

import string
import re
from typing import List, Union
from textarium.models import morph_ru, morph_en
import textarium.extraction as extraction

def remove_extra_spaces(text: str) -> str:
    """Removes extra spaces from a string

    Args:
        text (str): Any string

    Returns:
        str: A string without extra spaces (maximum 1 space between two words)

    Examples:
        >>> remove_extra_spaces("  This line    has extra   spaces   .  ")
        "This line has extra spaces."
    """
    text_wo_extra_spaces = " ".join(text.split())

    return text_wo_extra_spaces.strip()

def remove_charset(text: str, charset: string) -> str:
    """Removes provided chars from a string

    Args:
        text (str): Any string
        charset (str): A string of chars which should be removed

    Returns:
        str: A string without provided chars

    Examples:
        >>> remove_charset("I want to remove w and r from this line")
        "I ant to emove and fom this line"
    """
    text_wo_charset = text.translate(
        str.maketrans(charset, " " * len(charset))
    )
    text_wo_charset = remove_extra_spaces(text_wo_charset)

    return text_wo_charset

def remove_html(text: str) -> str:
    """Removes HTML-tags and other special symbols from a string

    Args:
        text (str): Any string

    Returns:
        str: A string without HTML-tags

    Examples:
        >>> remove_html("<div>A html-block with price</div>")
        "A html-block with price"
    """
    re_pattern = re.compile("<.*?>+")
    text = re_pattern.sub(r" ", text)
    text = text.replace("&nbsp", " ")
    text = text.replace("&lt", " ")
    text = text.replace("&gt", " ")
    text_wo_html_tags = remove_extra_spaces(text)
    return text_wo_html_tags

def remove_urls(text: str) -> str:
    """Removes URLs from a string

    Args:
        text (str): Any string

    Returns:
        str: A string without any URLs

    Examples:
        >>> remove_urls("Don't need links like http://google.com in my text.")
        "Don't need links like in my text."
    """
    url_pattern = re.compile(r"http\S+")
    text_wo_urls = url_pattern.sub(r"", text)
    text_wo_urls = remove_extra_spaces(text_wo_urls)
    return text_wo_urls

def remove_words(text: str, words_to_exclude: List) -> str:
    """Removes any particular words from a string (e.g. stopwords)

    Args:
        text (str): Any string
        words_to_exclude (List): A list of words to exclude from a string

    Returns:
        str: A string without excluded words

    Examples:
        >>> remove_words(
            "This line has too many unmeaningful words", 
            words_to_exclude=["this", "has"]
            )
        "line too many unmeaningful words"
    """
    tokens = extraction.extract_words(text)
    cleaned_tokens = [w for w in tokens if not w.lower() in words_to_exclude]
    return " ".join(cleaned_tokens)

def remove_digits(text: str) -> str:
    """Removes digits from a string

    Args:
        text (str): Any string

    Returns:
        str: A string without digits
    """
    text_wo_digits = text.translate(
        str.maketrans(string.digits, " " * len(string.digits))
    )
    text_wo_digits = remove_extra_spaces(text_wo_digits)
    return text_wo_digits

def remove_punctuation(text: str) -> str:
    """Removes punctuation from a string

    Args:
        text (str): Any string

    Returns:
        str: A string without punctuation
    """
    text_wo_punct = text.translate(
        str.maketrans(string.punctuation, " " * len(string.punctuation))
    )
    text_wo_punct = remove_extra_spaces(text_wo_punct)
    return text_wo_punct

def make_lowercase(text: str) -> str:
    """Makes text lowercase

    Args:
        text (str): Any string

    Returns:
        str: Lowercase text
    """
    return text.lower()

def lemmatize(text: Union[List, str], lang: str) -> str:
    """Lemmatizes all words in a text

    Args:
        text (Union[List, str]): Any string or a list of words
        morph (_type_): _description_

    Returns:
        str: Lemmatized text (string)
    """
    if type(text) == str:
        words = extraction.extract_words(text)
    elif type(text) == list:
        words = text
    
    if lang == 'ru':
        lemmatized_words = [morph_ru.parse(w)[0].normal_form for w in words]
    elif lang == 'en':
        lemmatized_words = [morph_en.lemmatize(w) for w in words]
    else:
        raise NotImplementedError("This package supports following languages: 'en' (english), 'ru' (russian).")

    return " ".join(lemmatized_words)


from unittest import TestCase

import textarium.extraction as extraction

class TestExtraction(TestCase):
    def test_extract_words_en_0(self):
        test_input = "This line has 5 tokens"
        expected_result = ["This", "line", "has", "5", "tokens"]
        result = extraction.extract_words(test_input)
        self.assertListEqual(expected_result, result)

    def test_extract_words_ru_0(self):
        test_input = "В этой строке 5 токенов"
        expected_result = ["В", "этой", "строке", "5", "токенов"]
        result = extraction.extract_words(test_input)
        self.assertListEqual(expected_result, result)

    def test_extract_sentences_en_0(self):
        text_input = """
        Hello! My name is Robbie. 
        Please, write an email to Mr. Parker. 
        His email: parker@gmail.com.
        """
        expected_result = [
            "Hello!", "My name is Robbie.", 
            "Please, write an email to Mr. Parker.", 
            "His email: parker@gmail.com."
        ]
        result = extraction.extract_sentences(text_input, lang='en')
        self.assertListEqual(expected_result, result)

    def test_extract_sentences_ru_0(self):
        text_input = """
        Привет! Меня зовут Робби.
        Пожалуйста, напиши письмо (т. е. e-mail) Паркеру.
        Его адрес: parker@gmail.com.
        """
        expected_result = [
            "Привет!", "Меня зовут Робби.",
            "Пожалуйста, напиши письмо (т. е. e-mail) Паркеру.",
            "Его адрес: parker@gmail.com."
        ]
        result = extraction.extract_sentences(text_input, lang='ru')
        self.assertListEqual(expected_result, result)

    def test_extract_urls_0(self):
        text_input = """
        There is one link: http://google.com.
        And another one: https://www.google.com/images?v=1&p=2!
        This is not a link: test@gmail.com
        """
        exptected_result = [
            "http://google.com",
            "https://www.google.com/images?v=1&p=2"
        ]
        result = extraction.extract_urls(text_input)
        self.assertListEqual(exptected_result, result)

    def test_extract_emails(self):
        text_input = """
        Please write to stan.dobryakov@gmail.com
        and add to cc another email of @sdobryakov: std@gmail.com
        """
        expected_result = [
            "stan.dobryakov@gmail.com",
            "std@gmail.com"
        ]
        result = extraction.extract_emails(text_input)
        self.assertListEqual(expected_result, result)

    def test_extract_dates(self):
        text_input = """
        The cerememony will take place on 21-02-2013
        despite of the fact that I was born on 15/04/1993
        """
        expected_result = ["2013-02-21T00:00:00", "1993-04-15T00:00:00"]
        result = [d.isoformat() for d in extraction.extract_dates(text_input)]
        self.assertListEqual(expected_result, result)

    def test_extract_ngrams(self):
        text_input = """
        this sentence consists of 8 bigrams and 7 trigrams
        """
        expected_result = [
            ("this", "sentence"),
            ("sentence", "consists"),
            ("consists", "of"),
            ("of", "8"),
            ("8", "bigrams"),
            ("bigrams", "and"),
            ("and", "7"),
            ("7", "trigrams")
        ]
        result = extraction.extract_ngrams(text_input, n=2)
        self.assertListEqual(expected_result, result)



from unittest import TestCase

from textarium.stats import text_stats as ts

class TestTextStats(TestCase):
    def test_characters_count(self):
        input = "This text contains 33 characters."
        expected_result = 33
        self.assertEqual(ts.characters_count(input), expected_result)

    def test_words_count(self):
        input = "This text contains 5 words."
        expected_result = 5
        self.assertEqual(ts.words_count(input), expected_result)

    def test_senteces_count(self):
        input = "Hi! I'm Jack. How are you doing?"
        expected_result = 3
        self.assertEqual(ts.sentences_count(input), expected_result)

    def test_urls_count(self):
        input = "What is better: http://google.com or https://www.yahoo.com?"
        expected_result = 2
        self.assertEqual(ts.urls_count(input), expected_result)

    def test_unique_words_count(self):
        input = "Hi! I am Jack. How are you doing? How was your day? Jack, what are your plans?"
        expected_result = 13
        self.assertEqual(ts.unique_words_count(input), expected_result)

    def test_characters_per_word(self):
        input = "This text contains 33 characters."
        expected_result = (4 + 4 + 8 + 10) / 4
        self.assertEqual(ts.characters_per_word(input), expected_result)

    def test_words_per_sentence(self):
        input = "Hi! I am Jack. How are you doing? How was your day? Jack, what are your plans?"
        expected_result = (1 + 3 + 4 + 4 + 5) / 5
        self.assertEqual(ts.words_per_sentence(input), expected_result)

    def test_count_words_by_predicat(self):
        input = "Hi! I am Jack. How are you doing? How was your day? Jack, what are your plans?"
        predicat_func = lambda x: len(x) <= 2
        expected_result = 3
        self.assertEqual(ts.count_words_by_predicat(input, predicat_func), expected_result)

    def test_long_words_count(self):
        input = "This sentence contains 2 long words"
        expected_result = 2
        self.assertEqual(ts.long_words_count(input), expected_result)

    def test_short_words_count(self):
        input = "This sentence contains 5 short words: i, am, one, two."
        expected_result = 5
        self.assertEqual(ts.short_words_count(input), expected_result)